# combinedIRFs.R
#
# Combines selected IRFS for the various models into one set of
# graphs.
#
# Patrick T. Brandt
# pbrandt@utdallas.edu
#
# 2006-03-03 Initial version
# 2006-03-10 Revised to present 3x3 results for IP-PI-JPI
# 2006-05-06 Revised to generate new subsets for model comparisons,
#            per Mike's e-mail of 2006-04-25.
# 2007-06-16 Revised to use latest results and models

library(MSBVAR)

# Load data -- just the impulse responses for each model and subset
# them to save memory / space.

# Split the responses into three blocks for different models
# Block A: IP, PI, AI, AP shock / responses for bystander and
#          credibility
# Block B: JPI responses to IP and PI for follower and credibility
# Block C: IP and PI responses for JPI shock for accountability and
#          credbility

block.A <- c(1:4, 8:11, 15:18, 22:25)
block.B <- c(7, 14)
block.C <- c(43:46)

load("bystander.AFP.means.exog.IRF-posdiag.RData")
bystander.A <- IRF.out[,,block.A]
rm(IRF.out)
gc()

load("follower.AFP.means.exog.IRF-posdiag.RData")
follower.B <- IRF.out[,,block.B]
rm(IRF.out)
gc()

load("accountability.AFP.means.exog.IRF-posdiag.RData")
accountability.C <- IRF.out[,,block.C]
rm(IRF.out)
gc()

load("credibility.AFP.means.exog.IRF-posdiag.RData")
credibility.A <- IRF.out[,,block.A]
credibility.B <- IRF.out[,,block.B]
credibility.C <- IRF.out[,,block.C]
rm(IRF.out)
gc()

# Make a list of the IRFs
irfs.A <- list(bystander.A, credibility.A)
irfs.B <- list(follower.B, credibility.B)
irfs.C <- list(accountability.C, credibility.C)

# Define the names for the equations

varnames <- c("I2P","P2I","A2I","A2P","I2A","P2A","JPI")

######################################################################
# New function that uses a list of MCMC generated impulse responses
# and knows how to combine them into one set of graphs with smart
# coloring / line types.  Only implements the Sims-Zha2 and Percentile
# methods.
######################################################################

plot.mc.irf.svar.subset <- function(mc.impulse.list, num.eqn, num.shk,
                                    probs = c(0.16, 0.84),
                                    eqnnames = NULL,
                                    shknames = NULL,
                                    method="Percentile",
                                    component=1,
                                    ...)

{
    total <- num.eqn*num.shk
    m <- num.eqn
    nsteps <- dim(mc.impulse.list[[1]])[2]
    draws <- dim(mc.impulse.list[[1]])[1]

    # Now build a set of lists of arrays to store the irf summaries
    irf.storage <- array(0, c(nsteps, length(probs) + 1, total,
                              length(mc.impulse.list)))

    irf.ci <- array(0, c(nsteps, length(probs) + 1, total))

    if (method == "Percentile") {
    for (j in 1:length(mc.impulse.list))
    { if (total>1){
        for (i in 1:total) {
            irf.bands <- t(apply(mc.impulse.list[[j]][, , i], 2, quantile,
                                 probs))
            irf.mean <- apply(mc.impulse.list[[j]][, , i], 2, median)
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    } else { for (i in 1:total) {
        irf.bands <- t(apply(mc.impulse.list[[j]], 2, quantile,
                             probs))
        irf.mean <- apply(mc.impulse.list[[j]], 2, median)
        irf.ci[, , i] <- cbind(irf.bands, irf.mean)
    }
         }
      irf.storage[,,,j] <- irf.ci
  }
}

    if (method == "Sims-Zha2") {
      for (j in 1:length(mc.impulse.list))
        {
          for (i in 1:total) {
             if (total>1) {
                 decomp <- eigen(var(mc.impulse.list[[j]][, , i]),
                                 symmetric = T)
                 W <- decomp$vectors
                 lambda <- decomp$values
                 gammak <- mc.impulse.list[[j]][, , i] * (W[component, ])
                 gammak.quantiles <- t(apply(gammak, 2, quantile,
                                             probs = probs))
                 irf.mean <- apply(mc.impulse.list[[j]][, , i], 2,
                                   median)
                 # print eigen %
                 cat("Eigen %", (100*lambda/sum(lambda))[1], "\n")

             } else {
                 decomp <- eigen(var(mc.impulse.list[[j]]),
                 symmetric=T)
                          W <- decomp$vectors
                 lambda <- decomp$values
                 gammak <- mc.impulse.list[[j]] * (W[component, ])
                 gammak.quantiles <- t(apply(gammak, 2, quantile,
                                             probs = probs))
                 irf.mean <- apply(mc.impulse.list[[j]], 2, median)

                 cat("Eigen %", (100*lambda/sum(lambda))[1], "\n")

             }

            irf.bands <- irf.mean + gammak.quantiles
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
         }

          irf.storage[,,,j] <- irf.ci
      }
    }

    # Clean up
    rm(irf.ci)
    gc()

    # Now find the min-max values for the plots
    minmax <- array(0, c(num.eqn, 2, length(mc.impulse.list)))

##     for (j in 1:length(mc.impulse.list))
##     {
##         within.plots <- apply(irf.storage[,,,j], 3, range)
##         tmp <- (c(1:total)%%num.eqn)
##         tmp[tmp == 0] <- num.eqn
##         indices <- sort(tmp, index.return = T)$ix
##         dim(indices) <- c(num.shk, num.eqn)
##         for (i in 1:num.eqn) {
##             minmax[i,,j] <- range(within.plots[, indices[, i]])
##         }
##     }

    if(num.eqn>1)
    {
        for (j in 1:length(mc.impulse.list))
        {
            within.plots <- apply(irf.storage[,,,j], 3, range)
            tmp <- (c(1:total)%%num.eqn)
            tmp[tmp == 0] <- num.eqn
            indices <- sort(tmp, index.return = T)$ix
            dim(indices) <- c(num.shk, num.eqn)
            for (i in 1:num.eqn) {
                minmax[i,,j] <- range(within.plots[, indices[, i]])
            }
        }
    }else{ minmax <- array(range(irf.storage), c(num.eqn, 2,
    length(mc.impulse.list))) }

    # Now do the plotting
    j <- 1
    par(mfcol = c(num.eqn, num.shk), mai = c(0.25, 0.25, 0.15, 0.25),
        omi = c(0.35, 0.75, 1, 0.15), cex=0.9)
    for (i in 1:total) {
        lims <- ifelse((i-num.eqn)%%num.eqn==0, num.eqn,
                       (i-num.eqn)%%num.eqn)

        tmp <- matrix(irf.storage[, , i,], nrow=nsteps,
                      ncol=3*dim(irf.storage)[4])

        ts.plot(tmp,
                gpars = list(xlab = "",
                lty=rep(seq(1:length(mc.impulse.list)), 1, each=3),
                ylab = "", #col=c(rep("black",3), rep("red", 3)),
                lwd=1,
                ylim = 1.1*range(minmax[lims,,])))
        abline(h = 0)
        if(i<=num.eqn)
        { mtext(eqnnames[i], side=2, line=3) }
        if((i-1)%%num.eqn==0)
        { mtext(shknames[j], side=3, line=2)
          j <- j+1
      }

    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
}


#############################################################

#############################################################
# Plot of responses to show reciprocity in the bystander and
# credibility models
#############################################################

pdf(file="reciprocity_irf_eigen.pdf", width=6, height=6.5)

plot.mc.irf.svar.subset(irfs.A, num.eqn=4, num.shk=4,
                        probs = c(0.16,0.84),
                        eqnnames = varnames[1:4],
                        shknames = varnames[1:4],
                        method="Sims-Zha2")
dev.off()

# Now do the same thing using percentiles
pdf(file="reciprocity_irf_percentile.pdf", width=6, height=6.5)

plot.mc.irf.svar.subset(irfs.A, num.eqn=4, num.shk=4,
                        probs = c(0.16,0.84),
                        eqnnames = varnames[1:4],
                        shknames = varnames[1:4],
                        method="Percentile")
dev.off()

#############################################################
# Plot of JPI response to I2P and P2I innovations for credibility
# and follower models.
#############################################################

pdf(file="JPIresponse_irf_eigen.pdf", width=4.5, height=4.5)

plot.mc.irf.svar.subset(irfs.B, num.eqn=1, num.shk=2,
                        probs = c(0.16,0.84),
                        eqnnames = varnames[c(7)],
                        shknames = varnames[1:2],
                        method="Sims-Zha2")
dev.off()

# Now do the same thing using percentiles
pdf(file="JPIresponse_irf_percentile.pdf", width=4.5, height=4.5)

plot.mc.irf.svar.subset(irfs.B, num.eqn=1, num.shk=2,
                        probs = c(0.16,0.84),
                        eqnnames = varnames[c(7)],
                        shknames = varnames[1:2],
                        method="Percentile")
dev.off()

#############################################################
# Plot of I2P and P2I responses to JPI innovations
#############################################################

pdf(file="IPPIresponse2JPI_irf_eigen.pdf", width=2.5, height=6)

plot.mc.irf.svar.subset(irfs.C, num.eqn=4, num.shk=1,
                        probs = c(0.16,0.84),
                        eqnnames = varnames[1:4],
                        shknames = varnames[c(7)],
                        method="Sims-Zha2")

dev.off()

# Now do the same thing using percentiles
pdf(file="IPPIresponse2JPI_irf_percentile.pdf", width=2.5, height=6)

plot.mc.irf.svar.subset(irfs.C, num.eqn=4, num.shk=1,
                        probs = c(0.16,0.84),
                        eqnnames = varnames[1:4],
                        shknames = varnames[c(7)],
                        method="Percentile")

dev.off()

# quit
q(save="no")
