# forecastcompare.R
# Generates BVAR forecasts with and without the JPI equation.
# Patrick Brandt
#
# 20071029 : Revised and simplified from earlier code

# Load library
library(MSBVAR)

# Load the data and parameters from setup.R
load("data_and_structure200705.RData")

# Fit models
full <- szbvar(window(Y.AFP.means, end=c(2005,3)), p=p,
               z=window(z2, end=c(2005,3)), lambda0=lambda0,
               lambda1=lambda1, lambda3=lambda3, lambda4=lambda4,
               lambda5=lambda5, mu5=mu5, mu6=mu6, qm=qm)

partial <- szbvar(window(Y.AFP.means[,1:6], end=c(2005,3)), p=p,
               z=window(z2, end=c(2005,3)), lambda0=lambda0,
               lambda1=lambda1, lambda3=lambda3, lambda4=lambda4,
               lambda5=lambda5, mu5=mu5, mu6=mu6, qm=qm)


# Forecast

full.forecast <- forecast(full, nsteps=9,
                      exog.fut=window(z2, start=c(2005, 4)))

partial.forecast <- forecast(partial, nsteps=9,
                         exog.fut=window(z2, start=c(2005, 4)))


# Extract out the forecasts and actual values.
full.forecast <- window(ts(full.forecast, start=c(1996,4), freq=12),
                        start=c(2005,4))

partial.forecast <- window(ts(partial.forecast, start=c(1996,4),
                              freq=12), start=c(2005,4))

actual.data <- window(Y.AFP.means, start=c(2005,4))

# Compute the forecast distributions / posterior samples

# Here's a function for the forecasting.  This is the same as in MSBVAR.
uc.forecast.VAR <- function (varobj, nsteps, burnin, gibbs, exog = NULL)
{
    y <- varobj$y
    ar.coefs <- varobj$ar.coefs
    intercept <- varobj$intercept
    A0 <- t(chol(varobj$mean.S))
    X <- varobj$X
    Y <- varobj$Y
    H0 <- varobj$H0
    S0 <- varobj$S0
    mu <- varobj$hyperp
    exog.coefs <- varobj$exog.coefs
    z <- varobj$z
    lambda0 <- varobj$prior[1]
    lambda1 <- varobj$prior[2]
    lambda3 <- varobj$prior[3]
    lambda4 <- varobj$prior[4]
    lambda5 <- varobj$prior[5]
    mu5 <- varobj$prior[6]
    mu6 <- varobj$prior[7]
    nu <- varobj$prior[8]
    prior <- varobj$prior.type
    num.exog <- varobj$num.exog
    qm <- varobj$qm
    ncoef <- nrow(varobj$Bhat)
    starttime <- date()
    p <- dim(ar.coefs)[3]
    m <- ncol(y)
    k <- m * nsteps
    capT <- nrow(y)
    yforc <- array(0, c(gibbs, nsteps, m))
    for (i in 1:(burnin + gibbs)) {
        epsilon.i <- matrix(rnorm(nsteps * m), nrow = nsteps,
            ncol = m)
        ytmp <- forecast.VAR(varobj, nsteps, A0 = A0, shocks = epsilon.i)
        if (i > burnin) {
            for (j in 1:m) {
                yforc[(i - burnin), (1:nsteps), j] <- ytmp[((capT +
                  1):(capT + nsteps)), j]
            }
        }
        Y.update <- ytmp[(capT - p + 1):nrow(ytmp), ]

        # Now build rhs -- start with an empty matrix
        X.update <- matrix(0, nsteps, m*p+1)
        # Add in the constant
        X.update[, (m*p+1)] <- matrix(1, nsteps, 1)
        # Put in the lagged y's
        for (j in 1:p) {
            X.update[1:nsteps, (m * (j - 1) + 1):(m * j)] <- matrix(Y.update[(p +
                1 - j):(nsteps + p - j), ], ncol = m)
        }
        # Put in exogenous coefficients if there are any.
        if (is.null(exog) == F) {
            X.update <- cbind(X.update, exog)
        }
        Y.update <- rbind(Y, ytmp[(capT + 1):nrow(ytmp), ])
        X.update <- rbind(X, X.update)
        XX.update <- crossprod(X.update)
        hstar.update <- H0 + XX.update
        B.update <- solve((hstar.update), (crossprod(X.update,
            Y.update) + H0[, 1:m]))
        S.update <- (S0 + crossprod(Y.update) + H0[1:m, 1:m] -
            t(B.update) %*% (hstar.update) %*% (B.update))/(capT +
            nsteps + nu - m - 1)
        hstarinv <- solve(hstar.update)
        vcv.Bh <- kronecker(S.update, hstarinv)
        df <- capT - m * p - m - 1 + nsteps
        wisharts <- rwishart(1, df, diag(m))
        Sigmat <- (chol(S.update))
        Sigma.Draw <- t(Sigmat) %*% (df * solve(matrix(wisharts,
            m, m))) %*% Sigmat
        sqrtwish <- t(chol(Sigma.Draw))
        bcoefs.covar <- t(chol(vcv.Bh))
        aplus <- matrix(B.update, ncol = 1) + bcoefs.covar %*%
            matrix(rnorm(nrow(bcoefs.covar)), ncol = 1)
        aplus <- matrix(aplus, ncol = m)
        aplus.coefs <- t(aplus[1:(m * p), ])
        dim(aplus.coefs) <- c(m, m, p)
        aplus.coefs <- aperm(aplus.coefs, c(2, 1, 3))
        intercept <- aplus[(m * p + 1), ]
        ar.coefs <- aplus.coefs
        A0 <- sqrtwish
        if (num.exog != 0) {
            exog.coefs <- aplus[(m * p + 2):nrow(aplus), ]
        }
        if (i%%500 == 0) {
            cat("Gibbs Iteration = ", i, "     \n")
            if (i <= burnin) {
                cat("(Still a burn-in draw.)\n")
            }
        }
    }
    endtime <- date()
    cat("Start time : ", starttime, "\n")
    cat("End time   : ", endtime, "\n")
    output <- list(forecast = yforc)
    attr(output, "class") <- c("forecast.VAR")
    return(output)
}

# Now compute the forecasts
full.post <- uc.forecast.VAR(full, nsteps=9, burnin=N1,
                             gibbs=N2,
                             exog=as.matrix(window(z2, start=c(2005, 4))))

partial.post <- uc.forecast.VAR(partial, nsteps=9, burnin=N1,
                                gibbs=N2,
                                exog=as.matrix(window(z2, start=c(2005, 4))))



# Modified plotting function -- this is a revised version of the one
# currently in MSBVAR.

"plot.forecasts" <-
function(x, cf.level, method=c("Sims-Zha2"), probs=c(0.16,0.84),
         component=1, xlab="", ylab="", ...)
{
    if(method=="Percentile"){
     m.forecast<-apply(x, 2, mean)
     quant<-apply(x, 2, quantile, probs=probs)
     vplus<-quant[1,]
     vminus<-quant[2,]
     forc.ci<-ts(t(rbind(m.forecast,vplus,vminus,cf.level)),
                 start=c(2005,4), freq=12)
 }
     if (method == "Sims-Zha2") {
         nsteps <- ncol(x)
         eigen.sum <- matrix(0, 1, nsteps)
#         for (i in 1:m^2) {
         decomp <- eigen(var(x), symmetric = T)
         W <- decomp$vectors
         lambda <- decomp$values
         gammak <- x * (W[component, ])
         gammak.quantiles <- t(apply(gammak, 2, quantile,
                                         probs = probs))
         m.forecast <- apply(x, 2, mean)
         quant <- m.forecast + gammak.quantiles
         forc.ci<-ts(cbind(m.forecast,quant, cf.level),
                     start=c(2005,4), freq=12)
         eigen.sum <- 100 * lambda/sum(lambda)
         cat("Eigen-vector percentages :", eigen.sum, "\n")

     }

     par(las=1)

     plot(forc.ci, ylab=ylab, xlab=xlab,
          axes=FALSE, plot.type="single", lty=c(rep(1,3), 2), ...)

     axis(1, at=c(2005.25, 2005.583, 2005.917),
          labels=c("April 2005","July 2005","Dec. 2005"))

     axis(2)
     box();
    }


# Plot the forecasts with 90% error bands
pdf(file="compareforecasts.pdf", width=9, height=4)

par(mfrow=c(2,7), mai = c(0.35, 0.25, 0.15, 0.15),
    omi = c(0.25, 1.3, 0.25, 0.15), cex.lab=0.8, cex.axis=0.8)

for(i in 1:7)
{
 plot.forecasts(full.post$forecast[,,i],
                cf.level=actual.data[,i],
                method="Sims-Zha2",
                probs=c(0.16,0.84),
                main=colnames(Y.AFP.means)[i])
 if(i==1) mtext("JPI equation", side=2, line=3, adj=1, cex=0.9)
 abline(h=0)
}


for(i in 1:6)
{
 plot.forecasts(partial.post$forecast[,,i],
                cf.level=actual.data[,i],
                method="Sims-Zha2",
                probs=c(0.16,0.84))

 if(i==1) mtext("No JPI equation", side=2, line=3, adj=1, cex=0.9)
 abline(h=0)
}

dev.off()


# Compute the forecast MSEs -- do this in two steps so we can separate
# out the bias from the variance in the forecasts

h <- 9

mean.full <- apply(full.post$forecast, c(2,3), mean)
mean.partial <- apply(partial.post$forecast, c(2,3), mean)

var.full <- apply(mean.full-actual.data, 2, var)
var.partial <- apply(mean.partial - actual.data[,1:6], 2, var)

bias2.full <- apply((mean.full - actual.data)^2, 2, mean)
bias2.partial <- apply((mean.partial - actual.data[,1:6])^2, 2, mean)

tmp <- cbind(var.full, c(var.partial, NA), bias2.full, c(bias2.partial, NA))

tmp <- cbind(tmp, sqrt(tmp[,1]+tmp[,3]), sqrt(tmp[,2]+tmp[,4]))
tmp <- cbind(tmp, tmp[,6]/tmp[,5], tmp[,4]/tmp[,3])

colnames(tmp) <- c("Variance with JPI", "Variance without JPI",
                   "Squared Bias with JPI", "Squared Bias without JPI",
                   "RMSE with JPI", "RMSE without JPI", "RMSE Ratio",
                   "Squared Bias ratio")

print(xtable(tmp, digits=rep(2, 9)))

## x <- sweep(full.post$forecast, c(2,3), actual.data)

## msefull <- array(0, c(m, m, h))
## for(i in 1:dim(x)[1])
## {
##     msetmp <- matrix(0, m, m)
##     for (j in 1:h)
##     {
##         msetmp <- msetmp + (1 + m*j/n)*tcrossprod(x[i,j,])
##         msefull[,,j] <- msefull[,,j] + msetmp
##     }

## }

## msefull <- msefull/dim(x)[1]

## print(msefull)

## y <- sweep(partial.post$forecast, c(2,3), actual.data[,1:6])

## msepartial <- array(0, c(m-1, m-1, h))
## for(i in 1:dim(y)[1])
## {
##     msetmp <- matrix(0, m-1,m-1)
##     for (j in 1:h)
##     {
##         msetmp <- msetmp + (1 + (m-1)*j/n)*tcrossprod(y[i,j,])
##         msepartial[,,j] <- msepartial[,,j] + msetmp
##     }

## }

## msepartial <- msepartial/dim(y)[1]

## print(msepartial)


## # Use the naive MSE estimators
## msefull.naive <- matrix(rowMeans(apply(x, 1, crossprod)/h), 7, 7)
## msepartial.naive <- matrix(rowMeans(apply(y, 1, crossprod)/h), 6, 6)

## print(msefull.naive)
## print(msepartial.naive)

## # Summarize
## tmp <- cbind(sqrt(diag(msefull.naive)), c(sqrt(diag(msepartial.naive)), NA))
## tmp <- cbind(tmp, 100*tmp[,1]/tmp[,2])

## print(xtable(tmp))

# Save
save.image("forecastcompare.RData")

# Quit
q(save="no")
