# samplemodels-recursive.R
# Draws the A(0) matrices from the posterior for each of the model
# specifications
#
# Patrick T. Brandt
# pbrandt@utdallas.edu
#
# 20070516 : Initial version
# 20070615 : Recursive models version

# Load libraries
library(MSBVAR)
library(Rmpi)       # RMPI interface
library(snow)       # SNOW system for multiple processors

# Load the data and fitted models
load("fittedmodels-recursive.RData")


# Model to sample using parallel chains
parallel.model <- function(i, fit.model, N1, N2, thin) {

    gibbs.A0(varobj=fit.model, N1=N1, N2=N2,
             thin=thin, normalization="PositiveDiagA")

}



# Define the index for the exogenous v. no exogenous variable models
exog.list <- c("exog", "nexog")

# Main loop to sample the models based on the specifications in
# setup.R and fitmodels.R.  Note that this code is setup for a dual
# processor machine since it draws two parallel MCMC chains of A(0)'s.

for(i in 1:length(model.list))
{
    for (j in 1:length(data.list))
    {
        for(k in 1:length(type.list))
        {
            for(l in 1:length(exog.list))
            {

                # Dynamically define the inputs and outputs for the models
                model.name <- paste(model.list[i], data.list[j],
                                    type.list[k], exog.list[l], sep=".")

                model.out <- paste(model.name, "A0parallel", sep=".")


                # Initialize cluster in R
                cl <- makeCluster(spec=2)
                clusterEvalQ(cl, library(MSBVAR))

                # Seed the RNG streams on each node of the cluster
                clusterSetupRNG(cl, seed=c(6093, 72871))




                # Draw A0 using parallel.model function in setup.R

                cat("-------------------------------------------------\n")
                cat("|Model name", model.name, "  |\n")
                cat("-------------------------------------------------\n")

                print(system.time(
                                  assign("model.out", clusterApply(cl, 1:2,
                                                                 parallel.model,
                                                                 fit.model=get(model.name),
                                                                 N1, N2, thin.iter))))

                stopCluster(cl)

                # Convert to an mcmc() object
                A0.free <- A0.chains2mcmc.list(model.out)

                # Save results

                save(model.out,
                     file=paste(model.name, "A0parallel.RData", sep=""),
                     compress=T)

                rm(model.out)

                # Now check convergence of the chains

                print(gelman.diag(A0.free))   # computes the psrf -- ratio of between to
                                              # within variance of the chains.

                if(N2>10000)  print(geweke.diag(A0.free))   # Geweke diagnostic
                print(effectiveSize(A0.free)) # Effective size of
                                              # posterior sample

                rm(A0.free)

            }
        }
    }
}


# Clean up and quit nicely

q(save="no")

