# setup.R
# Setup data and structure for B-SVAR models for
# Colaresi-Freeman-Brandt ISA paper
#
# Revision history
#
# 2006-02-05: initial version (PTB)
# 2006-02-19: Added the follower model specification
# 2006-03-02: Add common prior specification and the deterministic
# controls.
# 2006-03-05: Add dummy variables for ISR PM terms.
# 2006-03-07: Revised specification of the exogenous variables and
#             increased number of MCMC draws.
# 2006-03-26: Added data for the mean Goldstein scores.
# 2006-04-02: Added A0.chains function to this file.  Revised code to
# use MSBVAR 0.1.12
# 2006-04-06: Revised code to generate a set of endogenous variables
# for the means of Y
# 2006-04-07: Change the specification of the dummy variables for the
# election periods and the time trends (z2) to eliminate saturation of
# the first election period dummies.
# 2007-05-11: Revise the setup to work with updated KEDS / TABARI
# data.  Can now look at models for AFP and REU data in means and
# sums.
# 2007-05-29 : Use MSBVAR with C++ code to do longer runs
# 2007-06-13 : Corrected setup of exogenous and endogenous variables
# to be over same time intervals.
# 2007-10-19 : Revised to use latest version of MSBVAR

# Load packages
library(MSBVAR) ## Be sure you are using the latest version!

# Load the data file
load("AFP_and_REU_96-07Goldstein.RData")

# Now do some data cleanup / setup for the VAR models
#
# 1) Trim the sample to the point where the date coverages are same
#    for the KEDS and opinion data.  KEDS Levant data ends March 2005.
# 2) Add together the Palestinian actors --- including Hamas since
#    this is pre-2006
# 3) Build means and sums series for AFP and REU data.


# Trim data
data <- window(AFP.means.data, start=c(1996,4), end=c(2005, 12))

JPI.arima <- data[,"JPI"]

# Aggregate various actors
PAL2ISR <- data[,"PAL2ISR"] + data[,"HAMAS2ISR"] + data[,"ALAQ2ISR"]
PAL2USA <- data[,"PAL2USA"] + data[,"HAMAS2USA"] + data[,"ALAQ2USA"]
ISR2PAL <- data[,"ISR2PAL"] + data[,"ISR2HAMAS"] + data[,"ISR2ALAQ"]
ISR2USA <- data[,"ISR2USA"]
USA2ISR <- data[,"USA2ISR"]
USA2PAL <- data[,"USA2PAL"] + data[,"USA2HAMAS"] + data[,"USA2ALAQ"]

Y.AFP.means <- cbind(ISR2PAL, PAL2ISR,
                     USA2ISR, USA2PAL,
                     ISR2USA, PAL2USA,
                     JPI.arima)

colnames(Y.AFP.means) <- c("I2P","P2I","A2I","A2P","I2A","P2A","JPI")

# Trim data
data <- window(AFP.sums.data, start=c(1996,4), end=c(2005, 12))

# Aggregate various actors
PAL2ISR <- data[,"PAL2ISR"] + data[,"HAMAS2ISR"] + data[,"ALAQ2ISR"]
PAL2USA <- data[,"PAL2USA"] + data[,"HAMAS2USA"] + data[,"ALAQ2USA"]
ISR2PAL <- data[,"ISR2PAL"] + data[,"ISR2HAMAS"] + data[,"ISR2ALAQ"]
ISR2USA <- data[,"ISR2USA"]
USA2ISR <- data[,"USA2ISR"]
USA2PAL <- data[,"USA2PAL"] + data[,"USA2HAMAS"] + data[,"USA2ALAQ"]

Y.AFP.sums <- cbind(ISR2PAL, PAL2ISR,
                    USA2ISR, USA2PAL,
                    ISR2USA, PAL2USA,
                    JPI.arima)

colnames(Y.AFP.sums) <- c("I2P","P2I","A2I","A2P","I2A","P2A","JPI")

# Trim data
data <- window(REU.means.data, start=c(1996,4), end=c(2005, 12))

# Aggregate various actors
PAL2ISR <- data[,"PAL2ISR"] + data[,"HAMAS2ISR"] + data[,"ALAQ2ISR"]
PAL2USA <- data[,"PAL2USA"] + data[,"HAMAS2USA"]
ISR2PAL <- data[,"ISR2PAL"] + data[,"ISR2HAMAS"] + data[,"ISR2ALAQ"]
ISR2USA <- data[,"ISR2USA"]
USA2ISR <- data[,"USA2ISR"]
USA2PAL <- data[,"USA2PAL"] + data[,"USA2HAMAS"]

Y.REU.means <- cbind(ISR2PAL, PAL2ISR,
                     USA2ISR, USA2PAL,
                     ISR2USA, PAL2USA,
                     JPI.arima)

colnames(Y.REU.means) <- c("I2P","P2I","A2I","A2P","I2A","P2A","JPI")

# Trim data
data <- window(REU.sums.data, start=c(1996,4), end=c(2005, 12))

# Aggregate various actors
PAL2ISR <- data[,"PAL2ISR"] + data[,"HAMAS2ISR"] + data[,"ALAQ2ISR"]
PAL2USA <- data[,"PAL2USA"] + data[,"HAMAS2USA"]
ISR2PAL <- data[,"ISR2PAL"] + data[,"ISR2HAMAS"] + data[,"ISR2ALAQ"]
ISR2USA <- data[,"ISR2USA"]
USA2ISR <- data[,"USA2ISR"]
USA2PAL <- data[,"USA2PAL"] + data[,"USA2HAMAS"]

Y.REU.sums <- cbind(ISR2PAL, PAL2ISR,
                    USA2ISR, USA2PAL,
                    ISR2USA, PAL2USA,
                    JPI.arima)

colnames(Y.REU.sums) <- c("I2P","P2I","A2I","A2P","I2A","P2A","JPI")

# Clean up
rm(data)


# Test for lag lengths for the various series
print(var.lag.specification(Y.AFP.means, 12))
print(var.lag.specification(Y.AFP.sums, 12))

print(var.lag.specification(Y.REU.means, 12))
print(var.lag.specification(Y.REU.sums, 12))

# Looks like a two lag model will work. Chi^2 test picks 2 lags, AIC
# chooses two lags.

# Look at some Granger causality tests
granger.test(Y.AFP.means, 2)
granger.test(Y.AFP.sums, 2)
granger.test(Y.REU.means, 2)
granger.test(Y.REU.sums, 2)


# Now generate some election counters

## Here is a list of the Israeli elections from the last 20 years or so:
## 1 Nov. 1988
## 23 June 1992 (early)
## 29 May 1996 (early)
## 17 May 1999- First direct election of PM
## 6 Feb 2001-just for PM (due to Barak resignation), and last direct election
## 28 Jan 2003 (early)
## called for March 28 2006 (early)

# Y data start in 1996:4
ISR.election.counter <- ts(c(48, seq(1:36), seq(1:21), seq(1:23),
                             seq(1:36)), start=c(1996,4), freq=12)

n <- nrow(Y.AFP.means)

ISR.election1 <- ISR.election2 <- ISR.election3 <- ISR.election4 <-
    ISR.election <- matrix(0, n)

ISR.election1[1:37,] <- 1
ISR.election2[38:58,] <- 1
ISR.election3[59:81,] <- 1
ISR.election4[82:117,] <- 1

# Set up and estimate B-SVAR models
#
# Set up the A0 matrices.  Note that these are transposed from what we
# have in the notes and the papers, since there we present the A0 just
# as we would an IRF.  The structural.szbvar() function wants the
# equations in columns.

m <- ncol(Y.AFP.means)

# Bystander model
bystander.A0 <- diag(m)
colnames(bystander.A0) <- colnames(Y.AFP.means)
rownames(bystander.A0) <- colnames(Y.AFP.means)
bystander.A0[2:3,1] <- 1
bystander.A0[c(1,2,4),2] <- 1
bystander.A0[c(1,3,5),3] <- 1
bystander.A0[c(2,4,6),4] <- 1
bystander.A0[c(3,5),5] <- 1
bystander.A0[c(4,6),6] <- 1


# Follower model
follower.A0 <- bystander.A0
follower.A0[7,1] <- 1

# Accountability modeXl
accountability.A0 <- bystander.A0
accountability.A0[7,1:2] <- 1
accountability.A0[1,7] <- 1

# Credibility model
credibility.A0 <- accountability.A0
credibility.A0[1:4,7] <- 1

# Transpose all of the models identifications for the analysis.

bystander.A0 <- t(bystander.A0)
follower.A0 <- t(follower.A0)
accountability.A0 <- t(accountability.A0)
credibility.A0 <- t(credibility.A0)


# Add the various dummy variables and deterministic controls
# Time trends
# Data are April 1996 to March 2005
# Overall trend
t <- seq(1:n)
t2 <- t^2
t3 <- t^3

# 2nd Intifada trend -- begins Sept 28, 2000
intifada2 <- c(rep(0,54), seq(1:19), rep(0,44))

# Post Battle of Jenin trend, after April 12, 2002
postjenin <- c(rep(0,73), seq(1:44))

# Define matrices of exogenous variables

# Just Intifada2 and Jenin trends.
z <- as.matrix(cbind(intifada2, intifada2^2,
                     postjenin, postjenin^2))

# Dummies
intifada.dummy <- ifelse(intifada2>0, 1, 0)
postjenin.dummy <- ifelse(postjenin>0, 1, 0)

# Dummies and trends
z1 <- as.matrix(cbind(intifada2, intifada2^2, intifada.dummy,
                      postjenin, postjenin^2, postjenin.dummy))


# Omit the dummies for the first election period as a reference

# Dummies, elections, and interactions
z2 <- ts(as.matrix(cbind(intifada.dummy,
                      postjenin.dummy,
                      ISR.election2,
                      ISR.election3,
                      ISR.election4,
                      cumsum(ISR.election1)*ISR.election1,
                      cumsum(ISR.election2)*ISR.election2,
                      cumsum(ISR.election3)*ISR.election3,
                      cumsum(ISR.election4)*ISR.election4
                      )), start=c(1996,4), freq=12)


# Hyperparameters
p <- 2
lambda0 <- 0.8
lambda1 <- 0.1
lambda3 <- 1
lambda4 <- 0.1
lambda5 <- 0.05
mu5 <- 0
mu6 <- 5
qm <- 12


# Parameters for the Gibbs sampling
N1 <- 20000
N2 <- 100000
thin.iter <- 5


# Transform the A0's into a set of mcmc chains of the free parameters
# for posterior convergence analyses.

A0.chains2mcmc.list <- function(chains)
{ num.chains <- length(chains)

  chain.out <- vector(mode="list", length=num.chains)

  for (i in 1:num.chains)
  {
      chain.out[[i]] <- A02mcmc(chains[[i]])
  }

  return(mcmc.list(chain.out))
}

# Combine the parallel chains into one.
A0parallel2A0obj <- function(chains)
{ num.chains <- length(chains)
  A0.flat <- chains[[1]]$A0.posterior$A0
  A0.struct <- chains[[1]]$A0.posterior$struct
  m <- chains[[1]]$A0.posterior$m

  for(i in 2:num.chains)
  {
      A0.flat <- c(A0.flat, chains[[i]]$A0.posterior$A0)
  }

  A0.posterior <- list(A0=A0.flat,struct=A0.struct,m=m)

  return(list(A0.posterior=A0.posterior,
              ident=chains[[1]]$ident,
              thin=chains[[1]]$thin,
              N2=num.chains*chains[[1]]$N2))
}

# Save this as a file for later inputs / models

save.image("data_and_structure200705.RData")

q(save="no")
