library(zoo)

#file   - takes a filename or fully qualified URL
#actors - a matrix in which the first column in each row represents
#         the name of the aggregated dyad and the following columns
#         are the synonyms to aggregate
#dyads  - a matrix in which the first column represents sources
#         and the second column represents the targets of the dyads
#         you wish to keep (optional:  otherwise all dyads are
#         returned)
#freq   - the frequency of the time-series object


"keds.agg" <- function(file, actors, freq){

  # Read and parse KEDS tab delim WEIS encoded file
  eventdata <- read.table(file=file, sep="\t",
                          colClasses=c("character","character","character","character","NULL"))
  colnames(eventdata) <- c("date", "src", "tgt", "WEIS")
  print(nrow(eventdata))
  # Convert the dates into proper format (YYYY-MM-DD)
  eventdata[,1] <- as.Date(eventdata[,1], format="%y%m%d")

  # Read WEIS codes to Goldstein scores conversion table
  weis.dict <- read.table(file="WEISconversions.txt",
                          header=TRUE, sep="\t", quote="\"",
                          colClasses=c("numeric","character"))

  eventdata <- cbind(eventdata, rep(NA, nrow(eventdata)))

  # Convert WEIS codes
  for (i in 1:nrow(weis.dict))
      eventdata[eventdata$WEIS==weis.dict[i,2],5] <- weis.dict[i,1]

  eventdata[,5] <- as.numeric(eventdata[,5])
  colnames(eventdata) <- c("date", "src", "tgt", "WEIS", "goldstein")

  cat("WEIS codes converted\n")
  remove(weis.dict)

  # Recode actors
  srcs <- names(actors)
  tgts <- srcs

  for (i in 1:length(actors)){

      for(j in 1:length(actors[[i]])){
          eventdata[eventdata$src==actors[[i]][j],2] <- srcs[i]
          eventdata[eventdata$tgt==actors[[i]][j],3] <- tgts[i]
      }
  }
  cat("Actors recoded\n")

  # Aggregate dyad-days
  dyads.names <- vector(mode="character")

  # Create full.sample from starting date to ending date by frequency
  start <- min(as.Date(eventdata[,1]))
  cat(paste("\nStart Date:", start))
  start.year <- as.numeric(format(start, "%Y"))
  start.month <- as.numeric(format(start, "%m"))
  start.week <- as.numeric(format(start, "%U"))
  start.day <- as.numeric(format(start, "%d"))

  end <- max(as.Date(eventdata[,1]))
  end.year <- as.numeric(format(end, "%Y"))
  end.month <- as.numeric(format(end, "%m"))
  end.week <- as.numeric(format(end, "%U"))
  end.day <- as.numeric(format(end, "%d"))

  cat(paste("\nEnd Date:", end, "\n"))
  times <- seq(start, end, by="day")

  full.sample <- zoo(, times)
  dyad.num <- 0
  for (i in 1:length(srcs))
  { for (j in 1:length(tgts))
    {   if(i==j)  next;

        tmp <- na.omit(eventdata[eventdata$src==srcs[i] &
                                 eventdata$tgt==tgts[j],])

        if(nrow(tmp)<1) next;
            tmp.dyad <- aggregate(tmp$goldstein, by=list(src=tmp$src,
                                                 tgt=tmp$tgt,
                                                 date=tmp$date),
                                  sum)

            dyad.num <- dyad.num + 1
        cat("i is ", i, "j is ", j, "\n")

        dyad <- tmp.dyad[,3:4]
        dyad <- zoo(as.numeric(dyad[,2]), order.by=as.Date(dyad[,1]))

        dyad.name <- paste(srcs[i], "2", tgts[j], sep="")


        full.sample <- merge(full.sample, dyad, fill=0)#, suffixes=c("",dyad.name))
        cat(paste("Dyad #", dyad.num, ": ", dyad.name, " merged\n", sep=""))
#        dyads <- append(dyads, dyad)
        dyads.names <- append(dyads.names, dyad.name)

    }
  }

  # Name the columns
  colnames(full.sample) <- dyads.names

  # Clean up
  remove(eventdata)
  gc()

  if(freq=="day"){
      full.sample <- ts(full.sample, start=c(start.year,start.day), freq=365)
  }

  if(freq=="week"){
      weeks <- format(times, "%U")
      years <- format(times, "%Y")
      year.week <- paste(years, weeks, sep=".")
      full.sample <- ts(aggregate(as.matrix(full.sample), by=list(year.week),sum),
                        start=c(start.year,start.week), freq=52)
  }

  if(freq=="month"){
      months <- format(times, "%m")
      years <- format(times, "%Y")
      year.month <- paste(years, months, sep=".")
      full.sample <- ts(aggregate(as.matrix(full.sample), by=list(year.month), sum),
                         start=c(start.year,start.month), freq=12)
  }

  save( list=ls(all=TRUE), file=paste(file,"-sums.RData",sep=""), compress=TRUE)
  cat("\nAggregated Data Saved\n")

  # Drop the date column, since this is a TS object

  full.sample <- full.sample[,2:(ncol(full.sample))]

  return(full.sample)
}

# Files to be read

file.afp <- "~/BCF/Levant.WEIS.07.03/AFPLE.WEIS.evts.filt.clean"
file.reu <- "~/BCF/Levant.WEIS.07.03/REULE.WEIS.evts.filt.clean"

eventdata.afp <- read.table(file=file.afp, sep="\t",
                        colClasses=c("character","character","character","character","NULL"))

eventdata.reu <- read.table(file=file.reu, sep="\t",
                        colClasses=c("character","character","character","character","NULL"))


colnames(eventdata.afp) <- colnames(eventdata.reu) <- c("date", "src", "tgt", "WEIS")


# Define lists of the actors

actors <- vector(mode="list", length=6)
actors[[1]] <-
    c("ISR",   "ISRARB","ISRCOP",
      "ISRELI","ISRGOV","ISRJEW", "ISRJEWRAD","ISRJUD","ISRLAB","ISRMED",
      "ISRMIL","ISRMILJUD","ISRMILSPY","ISRMOSDRZ",
      "ISROPP","ISROPPLBA", "ISROPPSHA",
      "ISRSET","ISRSETRAD", "ISRSPY"
      )

actors[[2]] <- c("PAL", "PSE", "PSECHR","PSECOP","PSEELI","PSEGOV","PSEGOVFTA", "PSEGZS",
                 "PSEMED","PSEMIL","PSEMILJUD", "PSEMOS","PSEOPP",
                 "PSEREBANO", "PSEREBDFL", "PSEREBISJ", "PSEREBPFL",
                 "PSESPY","PSEWBN","PSEWBNREF", "PSEWSB","PSEWSBREF",
                 "PALCVL", "PALMED",    "PALOPP",    "PALPLO",
                 "PALREBPLF", "PALREF",
                 "PSERAD", "PSEREB"
                 )

actors[[3]] <- c("PSEREBHMS", "PSEREB","PSEGOVHMS" )

actors[[4]] <- c("PSEREBAAM")
actors[[5]] <-
    c("USA","USAARB","USABUS","USACHR", "USAEDU","USAELI","USAGOV","USAJEW","USAMED",
      "USAMIL","USAMOS","USAPOL","USASPY","USGOV")

actors[[6]] <- c("LBN","LBNCHR","LBNCHRCTH",
                 "LBNCHRMRN","LBNCHROPP","LBNCHRREB","LBNCOP",   "LBNELI",   "LBNGOV",
                 "LBNJUD",   "LBNMED",   "LBNMIL",   "LBNMOS",   "LBNMOSDRZ","LBNOPP",
                 "LBNPALREF", "LBNREB",   "LBNREBAML","LBNREBASL","LBNREBHEZ","LBNREBHZB",
                 "LBNREF",   "LBR")

names(actors) <- c("ISR", "PAL", "HAMAS", "ALAQ",  "USA", "LBN")

freq <- "month"

AFP.monthly <- keds.agg(file=file.afp, actors=actors, freq=freq)
REU.monthly <- keds.agg(file=file.reu, actors=actors, freq=freq)


# Save results

save(AFP.monthly, REU.monthly,
     file="LEVANT19912007MonthlySumsGoldstein.RData")

